{- Copyright (c) 2015, Peter Ljung <ljung.peter@gmail.com>

Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby
granted, provided that the above copyright notice and this permission notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL,
DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
USE OR PERFORMANCE OF THIS SOFTWARE. -}

import StockIo as Io
import StockView as V
import TimeSeries as TS
import Prelude as P
import StockIndicators
import Data.Colour
import Data.Colour.Names

{- Evaluation functional principles in stock analysis
   Compile: ghc -O2 FunStock.hs
   Optimize: strip FunStock -o FunStock
   Run: ./FunStock -}

main :: IO ()
main =
  do
    s0 <- loadShareFile "aapl"
    s1 <- loadShareFile "goog"
    let aapl = select Io.Close s0
    let goog = select Io.Close s1
    let ma10 = movingAvarage 10 aapl
    let var5 = movingVariance 5 aapl
    let gt110 = TS.filter (>110) aapl
    let std10 = movingDeviation 10 aapl
    let trMax = trailingMax aapl
    let trMin = trailingMin aapl
    let sTr = stopTrail 0.05 aapl
    let cross = crossesBelow ma10 aapl
    let var10 = movingVariance 10 goog
    let cov10 = movingCoVariance 10 (aapl, goog)
    let beta10 = movingBeta 10 (aapl, goog)
    let diff = differentiate aapl
    let indicators = [
                       ("AAPL", opaque blue, aapl)
                     , ("GOOG", opaque pink, goog)
                     , ("MA10", opaque red, ma10)
                     , ("VAR5", opaque pink, var5)
                     , ("GT110", opaque pink, gt110)
                     , ("SD10", opaque green, std10)
                     , ("MAX", opaque cyan, trMax)
                     , ("MIN", opaque red, trMin)
                     , ("STR5%", opaque purple, sTr)
                     , ("VAR10", opaque purple, TS.filter (<1000) var10)
                     , ("COV10", opaque purple, cov10)
                     , ("BETA10", opaque green, beta10)
                     , ("DIFF", opaque green, diff)
                     ]
    let signals = [ ("MA10 cross below", cross) ]
    draw "aapl.svg" indicators signals
    -- print $ show $ TS.drop 240 $ goog
    -- print $ show $ TS.drop 240 $ slidingWindow 10 goog
    -- print $ show $ TS.drop 240 var10
    return ()
