{- Copyright (c) 2015, Peter Ljung <ljung.peter@gmail.com>

Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby
granted, provided that the above copyright notice and this permission notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL,
DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
USE OR PERFORMANCE OF THIS SOFTWARE. -}

module StockView where
import Prelude as P
import TimeSeries as TS
import Data.Time.LocalTime
import Graphics.Rendering.Chart
import Graphics.Rendering.Chart.Backend.Diagrams
import Data.Default.Class
import Control.Lens
import Data.Colour

-- TODO: Let signals to be placed over an indicator

type TimeValue = (LocalTime,Float)   -- Datatype for diagrams with date on x-axis
type Name      = String
type FileName  = String

-- Values: x, y, spot size
asAreaSpots :: Signal -> [(LocalTime,Float,Float)]
asAreaSpots ts = P.map (\(x,_) -> (x,10,3)) $ P.filter snd $ asTimed ts

-- Make renderable from list of indicators and list of signals
asRenderable :: [(Name, AlphaColour Double, TimeSeries Float)] -> [(Name, Signal)] -> Renderable ()
asRenderable ts ss = toRenderable layout
  where
    layout = layout_title .~"Indicators & Signals"
           $ layout_plots .~ (line_plots ++ spots)
           $ def
    line_plots = P.map (toPlot . toLinePlot) ts
    toLinePlot (name, color, t) = plot_lines_style .~ lineStyle color
           $ plot_lines_values .~ [ asTimed t ]
           $ plot_lines_title .~ name
           $ def
    lineStyle color = line_width .~ 2
              $ line_color .~ color
              $ def
    spots = P.map (toPlot . toSpotPlot) ss
    toSpotPlot (name, s) = area_spots_title .~ name
          $ area_spots_max_radius .~ 10
          $ area_spots_values .~ asAreaSpots s
          $ def

-- Draw list of indicators and list of signals
draw :: FileName -> [(Name, AlphaColour Double, TimeSeries Float)] -> [(Name, Signal)] -> IO (PickFn ())
draw fname indicators signals = renderableToFile def fname $ asRenderable indicators signals
